/****************************************************************************************************************************
  UDPSendReceive.ino - Simple Arduino web server sample for ESP8266/ESP32 AT-command shield

  For Ethernet shields using WT32_ETH01 (ESP32 + LAN8720)

  WebServer_WT32_ETH01 is a library for the Ethernet LAN8720 in WT32_ETH01 to run WebServer

  Based on and modified from ESP8266 https://github.com/esp8266/Arduino/releases
  Built by Khoi Hoang https://github.com/khoih-prog/WebServer_WT32_ETH01
  Licensed under MIT license
 *****************************************************************************************************************************/

#define DEBUG_ETHERNET_WEBSERVER_PORT       Serial

// Debug Level from 0 to 4
#define _ETHERNET_WEBSERVER_LOGLEVEL_       3

#include <WebServer_WT32_ETH01.h>

// Select the IP address according to your local network
IPAddress myIP(10, 10, 10, 99);
IPAddress myGW(10, 10, 10, 1);
IPAddress mySN(255, 255, 255, 0);

// Google DNS Server IP
IPAddress myDNS(8, 8, 8, 8);

unsigned int localPort = 58430;  // local port to listen on

#define OUT_00_PIN 17
#define OUT_00_H REG_WRITE(GPIO_OUT_W1TS_REG, BIT17)
#define OUT_00_L REG_WRITE(GPIO_OUT_W1TC_REG, BIT17)
#define OUT_01_PIN 5
#define OUT_01_H REG_WRITE(GPIO_OUT_W1TS_REG, BIT5)
#define OUT_01_L REG_WRITE(GPIO_OUT_W1TC_REG, BIT5)
#define OUT_02_PIN 33
#define OUT_02_H REG_WRITE(GPIO_OUT1_W1TS_REG, BIT1) //33-32
#define OUT_02_L REG_WRITE(GPIO_OUT1_W1TC_REG, BIT1)
#define OUT_03_PIN 32
#define OUT_03_H REG_WRITE(GPIO_OUT1_W1TS_REG, BIT0) //32-32
#define OUT_03_L REG_WRITE(GPIO_OUT1_W1TC_REG, BIT0)
#define OUT_04_PIN 12
#define OUT_04_H REG_WRITE(GPIO_OUT_W1TS_REG, BIT12)
#define OUT_04_L REG_WRITE(GPIO_OUT_W1TC_REG, BIT12)
#define OUT_05_PIN 15
#define OUT_05_H REG_WRITE(GPIO_OUT_W1TS_REG, BIT15)
#define OUT_05_L REG_WRITE(GPIO_OUT_W1TC_REG, BIT15)

#define IN_00_PIN 1
#define IN_00 REG_READ(GPIO_IN_REG) & BIT1 //1
#define IN_01_PIN 3
#define IN_01 REG_READ(GPIO_IN_REG) & BIT3 //3
#define IN_02_PIN 39
#define IN_02 REG_READ(GPIO_IN1_REG) & BIT7 //39 -32
#define IN_03_PIN 36
#define IN_03 REG_READ(GPIO_IN1_REG) & BIT4 //36 -32
#define IN_04_PIN 14
#define IN_04 REG_READ(GPIO_IN_REG) & BIT14 //14
#define IN_05_PIN 35
#define IN_05 REG_READ(GPIO_IN1_REG) & BIT3 //35 -32
#define IN_06_PIN 4
#define IN_06 REG_READ(GPIO_IN_REG) & BIT4 //4
#define IN_07_PIN 2
#define IN_07 REG_READ(GPIO_IN_REG) & BIT2 //2

#define IO_00 0b00000001
#define IO_01 0b00000010
#define IO_02 0b00000100
#define IO_03 0b00001000
#define IO_04 0b00010000
#define IO_05 0b00100000
#define IO_06 0b01000000
#define IO_07 0b10000000

#define CTRL_READY    0b01000000
#define CTRL_ENABLE   0b10000000

char packetBuffer[255];          // buffer to hold incoming packet

// A UDP instance to let us send and receive packets over UDP
WiFiUDP Udp;

struct cmdPacket {
    uint8_t control;
    uint8_t io;
} cmd = { 0, 0 };

struct fbPacket {
    uint8_t control;
    uint8_t io;
} fb = { 0, 0 };

size_t send_udp_buffer(char *buffer, size_t size){
  size_t i;
  for(i=0;i<size;i++)
    Udp.write(buffer[i]);
  return i;
}

void setup()
{
    pinMode(OUT_00_PIN, OUTPUT);
    digitalWrite(OUT_00_PIN, 0);
    pinMode(OUT_01_PIN, OUTPUT);
    digitalWrite(OUT_01_PIN, 0);
    pinMode(OUT_02_PIN, OUTPUT);
    digitalWrite(OUT_02_PIN, 0);
    pinMode(OUT_03_PIN, OUTPUT);
    digitalWrite(OUT_03_PIN, 0);
    pinMode(OUT_04_PIN, OUTPUT);
    digitalWrite(OUT_04_PIN, 0);
    pinMode(OUT_05_PIN, OUTPUT);
    digitalWrite(OUT_05_PIN, 0);
        
    pinMode(IN_00_PIN, INPUT_PULLUP);
    pinMode(IN_01_PIN, INPUT_PULLUP);
    pinMode(IN_02_PIN, INPUT_PULLUP);
    pinMode(IN_03_PIN, INPUT_PULLUP);
    pinMode(IN_04_PIN, INPUT_PULLUP);
    pinMode(IN_05_PIN, INPUT_PULLUP);
    pinMode(IN_06_PIN, INPUT_PULLUP);
    pinMode(IN_07_PIN, INPUT_PULLUP);

  // To be called before ETH.begin()
  WT32_ETH01_onEvent();

  //bool begin(uint8_t phy_addr=ETH_PHY_ADDR, int power=ETH_PHY_POWER, int mdc=ETH_PHY_MDC, int mdio=ETH_PHY_MDIO,
  //           eth_phy_type_t type=ETH_PHY_TYPE, eth_clock_mode_t clk_mode=ETH_CLK_MODE);
  //ETH.begin(ETH_PHY_ADDR, ETH_PHY_POWER, ETH_PHY_MDC, ETH_PHY_MDIO, ETH_PHY_TYPE, ETH_CLK_MODE);
  ETH.begin(ETH_PHY_ADDR, ETH_PHY_POWER);

  // Static IP, leave without this line to get IP via DHCP
  //bool config(IPAddress local_ip, IPAddress gateway, IPAddress subnet, IPAddress dns1 = 0, IPAddress dns2 = 0);
  ETH.config(myIP, myGW, mySN, myDNS);

  WT32_ETH01_waitForConnect();

  Udp.begin(localPort);
}

void loop()
{
  // if there's data available, read a packet
  int packetSize = Udp.parsePacket();

  if (packetSize)
  {
    // read the packet into packetBufffer
    
    Udp.read(packetBuffer, sizeof(cmd) + 1);

    uint8_t chk = 71;
    for (int i = 0; i < sizeof(cmd); i++){
    chk ^= packetBuffer[i];
    }
    if (packetBuffer[sizeof(cmd)] == chk) {
    memcpy(&cmd, &packetBuffer, sizeof(cmd));
    
    if (cmd.control & CTRL_ENABLE) {
    fb.control |= CTRL_READY;   
    (cmd.io & IO_00) ? OUT_00_H : OUT_00_L;
    (cmd.io & IO_01) ? OUT_01_H : OUT_01_L;
    (cmd.io & IO_02) ? OUT_02_H : OUT_02_L;
    (cmd.io & IO_03) ? OUT_03_H : OUT_03_L;
    (cmd.io & IO_04) ? OUT_04_H : OUT_04_L;
    (cmd.io & IO_05) ? OUT_05_H : OUT_05_L;
            
    (IN_00) ? fb.io = IO_00 : fb.io = 0;
    if (IN_01)
        fb.io |= IO_01;
    if (IN_02)
        fb.io |= IO_02;
    if (IN_03)
        fb.io |= IO_03;
    if (IN_04)
        fb.io |= IO_04;
    if (IN_05)
        fb.io |= IO_05;
    if (IN_06)
        fb.io |= IO_06;
    if (IN_07)
        fb.io |= IO_07;
        
    memcpy(&packetBuffer, &fb, sizeof(fb));
    Udp.beginPacket(Udp.remoteIP(), Udp.remotePort());
    send_udp_buffer(packetBuffer, sizeof(packetBuffer));
    Udp.endPacket();
  }
  else fb.control = 0;
}
}
}
